#include "v3.h"

V3::V3(float x, float y, float z) {

	xyz[0] = x;
	xyz[1] = y;
	xyz[2] = z;

}

float& V3::operator[](int i) {

	return xyz[i];

}


V3 V3::operator-(V3 v1) {
	V3 ret;
	ret[0] = xyz[0] - v1[0];
	ret[1] = xyz[1] - v1[1];
	ret[2] = xyz[2] - v1[2];
	return ret;
}

V3 V3::operator+(V3 v1) {
	V3 ret;
	ret[0] = xyz[0] + v1[0];
	ret[1] = xyz[1] + v1[1];
	ret[2] = xyz[2] + v1[2];
	return ret;
}

ostream& operator<<(ostream &os, V3 v) {

	os << v[0] << " " << v[1] << " " << v[2];
	return os;

}

float V3::operator*(V3 v1) {

	V3 &v0 = *this;
	return v0[0] * v1[0] + v0[1] * v1[1] + v0[2] * v1[2];

}

float V3::Length() {

	V3 &v = *this;
	return sqrtf(v*v);


}

V3 V3::operator/(float scf) {

	V3 &v = *this;
	return V3(v[0] / scf, v[1] / scf, v[2] / scf);

}

V3 V3::operator*(float scf) {

	V3 &v = *this;
	return V3(v[0] * scf, v[1] * scf, v[2] * scf);

}

V3 V3::operator^(V3 v2) {

	V3 &v1 = *this;
	V3 ret(v1[1] * v2[2] - v1[2] * v2[1],
		v1[2] * v2[0] - v1[0] * v2[2],
		v1[0] * v2[1] - v1[1] * v2[0]);
	return ret;
}

unsigned int V3::GetColor() {

	V3 &v = *this;
	unsigned int ret = 0xFF000000;
	unsigned char *rgba = (unsigned char*)&ret;
	for (int i = 0; i < 3; i++) {
		int ichan = (int)(255.0f*v[i]);
		ichan = (ichan < 0) ? 0 : ichan;
		ichan = (ichan > 255) ? 255 : ichan;
		rgba[i] = ichan;
	}

	return ret;

}


void V3::SetColor(unsigned int color) {

	cerr << "ERROR: not yet implemented" << endl;

}

V3 V3::UnitVector() {

	return (*this) / Length();

}
